'use strict';

var React = require('react');
var _ = require('lodash');

var _require = require('yylib-ui'),
    YYClass = _require.YYClass,
    YYButton = _require.YYButton,
    YYTag = _require.YYTag,
    YYIcon = _require.YYIcon;

var formatUtils = require('yylib-utils/formatUtils');
var CPFilterRow = require('./CPFilterRow');
require('./index.less');

var PropTypes = YYClass.PropTypes;
var field = YYClass.field;
var uiMeta = {
    name: 'cp2-filterbar',
    fields: []
};

var CPFilterBar = YYClass.create({

    uiMeta: uiMeta,

    getInitialState: function getInitialState() {
        return {
            isExpanded: false,
            filterItems: []
        };
    },
    componentWillMount: function componentWillMount() {
        var _this = this;

        var filterItems = this.props.children;
        _.forEach(filterItems, function (filterItem) {
            _this.state.filterItems.push({
                rowKey: filterItem.props.rowKey,
                data: []
            });
        });
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
        var _this2 = this;

        if ("filterItems" in nextProps && nextProps.filterItems) {
            this.setState({ filterItems: nextProps.filterItems });
        } else if ("children" in nextProps && nextProps.children !== this.props.children) {
            var filterItems = nextProps.children;
            this.state.filterItems.length = 0;
            _.forEach(filterItems, function (filterItem) {
                _this2.state.filterItems.push({
                    rowKey: filterItem.props.rowKey,
                    data: []
                });
            });
        }
    },
    className: function className() {
        return this.uiMeta.name;
    },
    getTag: function getTag(data) {
        var tagStr = [];
        _.forEach(data, function (item) {
            tagStr.push(item.text);
        });
        return tagStr.join();
    },
    handleClickExpanded: function handleClickExpanded() {
        this.setState({
            isExpanded: !this.state.isExpanded
        });
    },
    handleFilterClick: function handleFilterClick(item, checked, rowKey) {
        var filterItems = this.state.filterItems;
        var linkTable = this.props.linkTable;

        var filterItem = _.find(filterItems, ['rowKey', rowKey]);
        if (checked) {
            var value = item.value;
            if (item.type === 'date') {
                if (_.isArray(item.value)) {
                    value = [formatUtils.formatDate(item.value[0], 'yyyy-MM-dd'), formatUtils.formatDate(item.value[1], 'yyyy-MM-dd')];
                } else {
                    value = formatUtils.formatDate(item.value, 'yyyy-MM-dd');
                }
            }
            var presetFunc = null;
            if (item.funcId && item.funcCode && item.funcName) {
                presetFunc = {
                    "funcId": item.funcId,
                    "funcCode": item.funcCode,
                    "funcName": item.funcName
                };
            }
            filterItem.data.push({
                text: item.text,
                type: item.type,
                value: value,
                compare: item.compare,
                presetFunc: presetFunc
            });
        }
        this.setState({ filterItems: filterItems });
        if (_.isFunction(this.props.onChange)) {
            this.props.onChange(this.state.filterItems, linkTable || null);
        }
    },
    handleFilterSubmit: function handleFilterSubmit(items, rowKey) {
        var filterItems = this.state.filterItems;
        var linkTable = this.props.linkTable;

        var filterItem = _.find(filterItems, ['rowKey', rowKey]);
        filterItem.data.push({ text: items.text, type: 'input', value: items.value, compare: items.compare });
        this.setState({ filterItems: filterItems });
        if (_.isFunction(this.props.onChange)) {
            this.props.onChange(this.state.filterItems, linkTable || null);
        }
    },
    handleTagClose: function handleTagClose(filterItem) {
        var filterItems = this.state.filterItems;
        var linkTable = this.props.linkTable;

        _.find(filterItems, filterItem).data = [];
        this.setState({ filterItems: filterItems });
        if (_.isFunction(this.props.onChange)) {
            this.props.onChange(this.state.filterItems, linkTable || null);
        }
    },
    renderCurrTags: function renderCurrTags() {
        var _this3 = this;

        var filterItems = this.state.filterItems;

        var tagViews = [];
        _.forEach(filterItems, function (filterItem, tagKey) {
            if (filterItem.data.length > 0) {
                tagViews.push(React.createElement(
                    YYTag,
                    { key: "filter_tag_" + tagKey, closable: true, afterClose: function afterClose() {
                            return _this3.handleTagClose(filterItem);
                        } },
                    _this3.getTag(filterItem.data)
                ));
            }
        });
        return tagViews.length ? tagViews : [];
    },
    renderFilterViews: function renderFilterViews() {
        var _this4 = this;

        var filterItems = this.props.children;
        var _filterItems = this.state.filterItems;
        var filterViews = [];
        _.forEach(filterItems, function (filterItem, index) {
            if (_.find(_filterItems, ['rowKey', filterItem.props.rowKey]).data.length == 0) {
                filterViews.push(React.createElement(CPFilterRow, {
                    key: 'filter_row_' + index,
                    rowKey: filterItem.props.rowKey,
                    title: filterItem.props.title,
                    items: filterItem.props.children,
                    multiple: filterItem.props.multiple,
                    code: filterItem.props.code,
                    dataparams: filterItem.props.dataparams,
                    serverUrl: filterItem.props.serverUrl,
                    enableOrg: filterItem.props.enableOrg,
                    orgId: filterItem.props.orgId,
                    refId: filterItem.props.refId,
                    sysType: filterItem.props.sysType,
                    onClick: _this4.handleFilterClick,
                    onSubmit: _this4.handleFilterSubmit,
                    minCount: _this4.props.minCount
                }));
            }
        });
        return filterViews.length ? filterViews : null;
    },
    render: function render() {
        var isExpanded = this.state.isExpanded;

        var currTags = this.renderCurrTags();
        return React.createElement(
            'div',
            { className: this.className() },
            React.createElement(
                'div',
                { className: this.className() + '-selected' },
                React.createElement(
                    'span',
                    null,
                    '\u5F53\u524D\u7B5B\u9009\uFF1A'
                ),
                currTags,
                React.createElement(
                    YYButton,
                    { ghost: true, className: this.className() + "-showbtn", onClick: this.handleClickExpanded },
                    isExpanded ? '收起筛选' : '展开筛选',
                    React.createElement(YYIcon, { type: isExpanded ? 'up' : 'down' })
                )
            ),
            isExpanded ? this.renderFilterViews() : null
        );
    }
});

module.exports = CPFilterBar;